@@ -35,22 +35,27 @@ namespace Twitterizer.Core
 {
     using System;
     using System.Collections.Generic;
-    using System.Diagnostics;
     using System.Globalization;
-    using System.IO;
     using System.Net;
+    using System.Linq;
     using System.Text;
+#if !SILVERLIGHT
     using System.Web;
+#endif
+#if !LITE && !SILVERLIGHT
     using System.Web.Caching;
+#endif
     using Twitterizer;
 
     /// <summary>
     /// The base command class.
     /// </summary>
     /// <typeparam name="T">The business object the command should return.</typeparam>
+#if !SILVERLIGHT
     [Serializable]
+#endif
     internal abstract class TwitterCommand<T> : ICommand<T>
-        where T : class, ITwitterObject, new()
+        where T : ITwitterObject
     {
         /// <summary>
         /// Initializes a new instance of the <see cref="TwitterCommand&lt;T&gt;"/> class.
@@ -61,10 +66,10 @@ namespace Twitterizer.Core
         /// <param name="optionalProperties">The optional properties.</param>
         protected TwitterCommand(HTTPVerb method, string endPoint, OAuthTokens tokens, OptionalProperties optionalProperties)
         {
-            this.RequestParameters = new Dictionary<string, string>();
+			this.RequestParameters = new Dictionary<string, object>();
             this.Verb = method;
             this.Tokens = tokens;
-            this.OptionalProperties = optionalProperties == null ? new OptionalProperties() : optionalProperties;
+            this.OptionalProperties = optionalProperties ?? new OptionalProperties();
 
             this.SetCommandUri(endPoint);
         }
@@ -91,13 +96,7 @@ namespace Twitterizer.Core
         /// Gets or sets the request parameters.
         /// </summary>
         /// <value>The request parameters.</value>
-        public Dictionary<string, string> RequestParameters { get; set; }
-
-        /// <summary>
-        /// Gets or sets the image to upload.
-        /// </summary>
-        /// <value>The image to upload.</value>
-        public TwitterImage ImageToUpload { get; set; }
+        public Dictionary<string, object> RequestParameters { get; set; }
 
         /// <summary>
         /// Gets or sets the serialization delegate.
@@ -116,19 +115,25 @@ namespace Twitterizer.Core
         /// </summary>
         public abstract void Init();
 
+        /// <summary>
+        /// Gets or sets a value indicating whether this <see cref="TwitterCommand&lt;T&gt;"/> is multipart.
+        /// </summary>
+        /// <value><c>true</c> if multipart; otherwise, <c>false</c>.</value>
+        public bool Multipart { get; set; }
+
         /// <summary>
         /// Executes the command.
         /// </summary>
         /// <returns>The results of the command.</returns>
-        public T ExecuteCommand()
+        public TwitterResponse<T> ExecuteCommand()
         {
+            TwitterResponse<T> twitterResponse = new TwitterResponse<T>();
+
             if (this.OptionalProperties.UseSSL)
             {
                 this.Uri = new Uri(this.Uri.AbsoluteUri.Replace("http://", "https://"));
             }
 
-            Debug.WriteLine(string.Format(CultureInfo.CurrentCulture, "Begin {0}", this.Uri.AbsoluteUri), "Twitterizer2");
-
             // Loop through all of the custom attributes assigned to the command class
             foreach (Attribute attribute in this.GetType().GetCustomAttributes(false))
             {
@@ -150,7 +155,7 @@ namespace Twitterizer.Core
                 else if (attribute.GetType() == typeof(RateLimitedAttribute))
                 {
                     // Get the rate limiting status
-                    if (TwitterRateLimitStatus.GetStatus(this.Tokens).RemainingHits == 0)
+                    if (TwitterRateLimitStatus.GetStatus(this.Tokens).ResponseObject.RemainingHits == 0)
                     {
                         throw new TwitterizerException("You are being rate limited.");
                     }
@@ -158,6 +163,7 @@ namespace Twitterizer.Core
 
             }
 
+#if !LITE && !SILVERLIGHT
             // Variables and objects needed for caching
             StringBuilder cacheKeyBuilder = new StringBuilder(this.Uri.AbsoluteUri);
             if (this.Tokens != null)
@@ -166,42 +172,48 @@ namespace Twitterizer.Core
             }
 
             Cache cache = HttpRuntime.Cache;
+#endif
 
             // Prepare the query parameters
-            Dictionary<string, string> queryParameters = new Dictionary<string, string>();
-            foreach (KeyValuePair<string, string> item in this.RequestParameters)
+			Dictionary<string, object> queryParameters = new Dictionary<string, object>();
+			foreach (KeyValuePair<string, object> item in this.RequestParameters)
             {
                 queryParameters.Add(item.Key, item.Value);
+#if !LITE && !SILVERLIGHT
                 cacheKeyBuilder.AppendFormat("|{0}={1}", item.Key, item.Value);
+#endif
             }
 
+#if !LITE && !SILVERLIGHT
             // Lookup the cached item and return it
             if (this.Verb == HTTPVerb.GET && this.OptionalProperties.CacheOutput && cache[cacheKeyBuilder.ToString()] != null)
             {
                 if (cache[cacheKeyBuilder.ToString()] is T)
                 {
-                    Debug.WriteLine("Found in cache", "Twitterizer2");
-                    Debug.WriteLine(string.Format(CultureInfo.CurrentCulture, "End {0}", this.Uri.AbsoluteUri), "Twitterizer2");
-                    return (T)cache[cacheKeyBuilder.ToString()];
+                    return new TwitterResponse<T>()
+                               {
+                                   ResponseObject = (T)cache[cacheKeyBuilder.ToString()],
+                                   ResponseCached = true
+                               };
                 }
             }
-
+            
+#endif
             // Declare the variable to be returned
-            T resultObject = default(T);
-            RequestStatus requestStatus = null;
-            RateLimiting rateLimiting = null;
-
-            // This must be set for all twitter request.
-            System.Net.ServicePointManager.Expect100Continue = false;
-
-            byte[] responseData = null;
+            twitterResponse.ResponseObject = default(T);
+            twitterResponse.RequestUrl = this.Uri.AbsoluteUri;
+            RateLimiting rateLimiting;
+            AccessLevel accessLevel;
+            byte[] responseData;
 
             try
             {
-                WebRequestBuilder requestBuilder = new WebRequestBuilder(this.Uri, this.Verb, this.Tokens);
+				WebRequestBuilder requestBuilder = new WebRequestBuilder(this.Uri, this.Verb, this.Tokens, "") { Multipart = this.Multipart };
 
+#if !SILVERLIGHT
                 if (this.OptionalProperties != null)
                     requestBuilder.Proxy = this.OptionalProperties.Proxy;
+#endif
 
                 foreach (var item in queryParameters)
                 {
@@ -211,80 +223,132 @@ namespace Twitterizer.Core
                 HttpWebResponse response = requestBuilder.ExecuteRequest();
 
                 responseData = ConversionUtility.ReadStream(response.GetResponseStream());
+                twitterResponse.Content = Encoding.UTF8.GetString(responseData, 0, responseData.Length);
+
+                twitterResponse.RequestUrl = requestBuilder.RequestUri.AbsoluteUri;
 
                 // Parse the rate limiting HTTP Headers
                 rateLimiting = ParseRateLimitHeaders(response.Headers);
 
-                // Build the request status (holds details about the last request) and update the singleton
-                requestStatus = RequestStatus.BuildRequestStatus(
-                    responseData,
-                    response.ResponseUri.AbsoluteUri,
-                    response.StatusCode,
-                    response.ContentType,
-                    rateLimiting);
+                // Parse Access Level
+                accessLevel = ParseAccessLevel(response.Headers);
 
-                RequestStatus.UpdateRequestStatus(requestStatus);
+                // Lookup the status code and set the status accordingly
+                SetStatusCode(twitterResponse, response.StatusCode, rateLimiting);
+
+                twitterResponse.RateLimiting = rateLimiting;
+                twitterResponse.AccessLevel = accessLevel;
             }
             catch (WebException wex)
             {
-                Trace.TraceError(wex.Message);
+                if (new[]
+                        {
+#if !SILVERLIGHT
+                            WebExceptionStatus.Timeout, 
+                            WebExceptionStatus.ConnectionClosed,
+#endif
+                            WebExceptionStatus.ConnectFailure
+                        }.Contains(wex.Status))
+                {
+                    twitterResponse.Result = RequestResult.ConnectionFailure;
+                    twitterResponse.ErrorMessage = wex.Message;
+                    return twitterResponse;
+                }
 
                 // The exception response should always be an HttpWebResponse, but we check for good measure.
                 HttpWebResponse exceptionResponse = wex.Response as HttpWebResponse;
 
-                if (wex.Response == null)
+                if (exceptionResponse == null)
                 {
                     throw;
                 }
 
                 responseData = ConversionUtility.ReadStream(exceptionResponse.GetResponseStream());
+                twitterResponse.Content = Encoding.UTF8.GetString(responseData, 0, responseData.Length);
 
                 rateLimiting = ParseRateLimitHeaders(exceptionResponse.Headers);
 
-                requestStatus = RequestStatus.BuildRequestStatus(
-                        responseData,
-                        exceptionResponse.ResponseUri.AbsoluteUri,
-                        exceptionResponse.StatusCode,
-                        exceptionResponse.ContentType,
-                        rateLimiting);
+                // Parse Access Level
+                accessLevel = ParseAccessLevel(exceptionResponse.Headers);
 
-                RequestStatus.UpdateRequestStatus(requestStatus);
+                // Try to read the error message, if there is one.
+                try
+                {
+                    twitterResponse.ErrorMessage = SerializationHelper<TwitterErrorDetails>.Deserialize(responseData).ErrorMessage;
+                }
+                catch (Exception)
+                {
+                    // Occasionally, Twitter responds with XML error data even though we asked for json.
+                    // This is that scenario. We will deal with it by doing nothing. It's up to the developer to deal with it.
+                }
+
+                // Lookup the status code and set the status accordingly
+                SetStatusCode(twitterResponse, exceptionResponse.StatusCode, rateLimiting);
+
+                twitterResponse.RateLimiting = rateLimiting;
+                twitterResponse.AccessLevel = accessLevel;
 
                 if (wex.Status == WebExceptionStatus.UnknownError)
                     throw;
 
-                return new T() { IsEmpty = true, RequestStatus = requestStatus, RateLimiting = rateLimiting };
+                return twitterResponse;
             }
-            finally
+
+            try
             {
-                // Set this back to the default so it doesn't affect other .net code.
-                System.Net.ServicePointManager.Expect100Continue = true;
+                twitterResponse.ResponseObject = SerializationHelper<T>.Deserialize(responseData, this.DeserializationHandler);
+            }
+            catch (Newtonsoft.Json.JsonReaderException)
+            {
+                twitterResponse.ErrorMessage = "Unable to parse JSON";
+                twitterResponse.Result = RequestResult.Unknown;
+                return twitterResponse;
             }
 
-            resultObject = SerializationHelper<T>.Deserialize(responseData, this.DeserializationHandler);
-
-            this.AddResultToCache(cacheKeyBuilder, cache, resultObject);
-
-            if (resultObject == null)
-                resultObject = new T();
+#if !LITE && !SILVERLIGHT
+            this.AddResultToCache(cacheKeyBuilder, cache, twitterResponse.ResponseObject);
+#endif
 
             // Pass the current oauth tokens into the new object, so method calls from there will keep the authentication.
-            resultObject.Tokens = this.Tokens;
-            resultObject.RequestStatus = requestStatus;
-            resultObject.RateLimiting = rateLimiting;
+            twitterResponse.Tokens = this.Tokens;
 
-            Debug.WriteLine(string.Format(CultureInfo.CurrentCulture, "Finished {0}", this.Uri.AbsoluteUri), "Twitterizer2");
-
-            return resultObject;
+            return twitterResponse;
         }
 
         /// <summary>
-        /// Clones this instance.
+        /// Sets the status code.
         /// </summary>
-        /// <returns>A new instance of the <see cref="Twitterizer.Core.PagedCommand{T}"/> interface.</returns>
-        internal virtual TwitterCommand<T> Clone()
+        /// <param name="twitterResponse">The twitter response.</param>
+        /// <param name="statusCode">The status code.</param>
+        /// <param name="rateLimiting">The rate limiting.</param>
+        private static void SetStatusCode(TwitterResponse<T> twitterResponse, HttpStatusCode statusCode, RateLimiting rateLimiting)
         {
-            return default(TwitterCommand<T>);
+            switch (statusCode)
+            {
+                case HttpStatusCode.OK:
+                    twitterResponse.Result = RequestResult.Success;
+                    break;
+
+                case HttpStatusCode.BadRequest:
+                    twitterResponse.Result = rateLimiting.Remaining == 0 ? RequestResult.RateLimited : RequestResult.BadRequest;
+                    break;
+
+                case HttpStatusCode.Unauthorized:
+                    twitterResponse.Result = RequestResult.Unauthorized;
+                    break;
+
+                case HttpStatusCode.NotFound:
+                    twitterResponse.Result = RequestResult.FileNotFound;
+                    break;
+
+                case HttpStatusCode.ProxyAuthenticationRequired:
+                    twitterResponse.Result = RequestResult.ProxyAuthenticationRequired;
+                    break;
+
+                default:
+                    twitterResponse.Result = RequestResult.Unknown;
+                    break;
+            }
         }
 
         /// <summary>
@@ -293,6 +357,9 @@ namespace Twitterizer.Core
         /// <param name="endPoint">The end point.</param>
         protected void SetCommandUri(string endPoint)
         {
+            if (endPoint.StartsWith("/"))
+                throw new ArgumentException("The API endpoint cannot begin with a forward slash. This will result in 404 errors and headaches.", "endPoint");
+
             this.Uri = new Uri(string.Concat(this.OptionalProperties.APIBaseAddress, endPoint));
         }
 
@@ -305,24 +372,53 @@ namespace Twitterizer.Core
         {
             RateLimiting rateLimiting = new RateLimiting();
 
-            if (!string.IsNullOrEmpty(responseHeaders.Get("X-RateLimit-Limit")))
+            if (responseHeaders.AllKeys.Contains("X-RateLimit-Limit"))
             {
-                rateLimiting.Total = int.Parse(responseHeaders.Get("X-RateLimit-Limit"), CultureInfo.InvariantCulture);
+                rateLimiting.Total = int.Parse(responseHeaders["X-RateLimit-Limit"], CultureInfo.InvariantCulture);
             }
 
-            if (!string.IsNullOrEmpty(responseHeaders.Get("X-RateLimit-Remaining")))
+            if (responseHeaders.AllKeys.Contains("X-RateLimit-Remaining"))
             {
-                rateLimiting.Remaining = int.Parse(responseHeaders.Get("X-RateLimit-Remaining"), CultureInfo.InvariantCulture);
+                rateLimiting.Remaining = int.Parse(responseHeaders["X-RateLimit-Remaining"], CultureInfo.InvariantCulture);
             }
 
             if (!string.IsNullOrEmpty(responseHeaders["X-RateLimit-Reset"]))
             {
-                rateLimiting.ResetDate = TimeZone.CurrentTimeZone.ToLocalTime(DateTime.SpecifyKind(new DateTime(1970, 1, 1, 0, 0, 0, 0)
-                    .AddSeconds(double.Parse(responseHeaders.Get("X-RateLimit-Reset"), CultureInfo.InvariantCulture)), DateTimeKind.Utc));
+                rateLimiting.ResetDate = DateTime.SpecifyKind(new DateTime(1970, 1, 1, 0, 0, 0, 0)
+                    .AddSeconds(double.Parse(responseHeaders["X-RateLimit-Reset"], CultureInfo.InvariantCulture)), DateTimeKind.Utc);
             }
             return rateLimiting;
         }
 
+        /// <summary>
+        /// Parses the access level headers.
+        /// </summary>
+        /// <param name="responseHeaders">The headers of the web response.</param>
+        /// <returns>An enum of the current access level of the OAuth Token being used.</returns>
+        private AccessLevel ParseAccessLevel(WebHeaderCollection responseHeaders)
+        {
+            if (responseHeaders.AllKeys.Contains("X-Access-Level"))
+            {
+                switch (responseHeaders["X-Access-Level"].ToLower())
+                {
+                    case "read":
+                        return AccessLevel.Read;
+                    case "read-write":
+                        return AccessLevel.ReadWrite;
+                    case "read-write-privatemessages":
+                    case "read-write-directmessages":
+                        return AccessLevel.ReadWriteDirectMessage;
+                    default:
+                        break;
+                }
+                return AccessLevel.Unknown;
+            }
+            else
+                return AccessLevel.Unavailable;
+        }
+
+
+#if !LITE && !SILVERLIGHT
         /// <summary>
         /// Adds the result to cache.
         /// </summary>
@@ -338,13 +434,12 @@ namespace Twitterizer.Core
                     cacheKeyBuilder.ToString(),
                     resultObject,
                     null,
-                    Cache.NoAbsoluteExpiration,
-                    this.OptionalProperties.CacheTimespan,
+                    DateTime.Now.Add(this.OptionalProperties.CacheTimespan),
+                    Cache.NoSlidingExpiration,
                     CacheItemPriority.Normal,
                     null);
-
-                Debug.WriteLine(string.Format(CultureInfo.CurrentCulture, "Added results to cache", this.Uri.AbsoluteUri), "Twitterizer2");
             }
         }
-    }
+#endif
+        }
 }
