@@ -26,11 +26,11 @@ namespace Smuxi.Common
 {
     public static class Defines
     {
-        public const string GitBranch   = "master";
-        public const string GitCommitHash = "8312f5b";
+        public const string GitBranch   = "release/0.8.9";
+        public const string GitCommitHash = "fe845eb";
 
         private static readonly string f_InstallPrefix = "/usr/local";
-        private static readonly string f_DistVersion = "master/8312f5b";
+        private static readonly string f_DistVersion = "release/0.8.9/fe845eb";
         private static readonly string f_TwitterApiKey = "60QV2qQx9cS7y1BJDbgAA|2VgD6qQKddsF5HYQ0TrRgs3tFTnCwDONBmRlTmG658";
 
         public static string InstallPrefix {
