@@ -1,8 +1,6 @@
-// $Id$
-// 
 // Smuxi - Smart MUltipleXed Irc
 // 
-// Copyright (c) 2010 Mirco Bauer <meebey@meebey.net>
+// Copyright (c) 2010-2011 Mirco Bauer <meebey@meebey.net>
 // 
 // Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
 // 
@@ -152,9 +150,27 @@ namespace Smuxi.Common
                     Environment.SpecialFolder.LocalApplicationData
                 );
                 logPath = Path.Combine(logPath, "smuxi");
+                // FIXME: include session username
                 logPath = Path.Combine(logPath, "logs");
                 return logPath;
             }
         }
+
+        public static string GetBuffersPath(string username)
+        {
+            var dbPath = GetBuffersBasePath();
+            dbPath = Path.Combine(dbPath, IOSecurity.GetFilteredPath(username));
+            return dbPath;
+        }
+
+        public static string GetBuffersBasePath()
+        {
+            var dbPath = Environment.GetFolderPath(
+                Environment.SpecialFolder.LocalApplicationData
+            );
+            dbPath = Path.Combine(dbPath, "smuxi");
+            dbPath = Path.Combine(dbPath, "buffers");
+            return dbPath;
+        }
     }
 }
