@@ -1,6 +1,6 @@
 // Smuxi - Smart MUltipleXed Irc
 // 
-// Copyright (c) 2009-2010 Mirco Bauer <meebey@meebey.net>
+// Copyright (c) 2009-2011 Mirco Bauer <meebey@meebey.net>
 // 
 // Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
 // 
@@ -26,6 +26,7 @@ using System.Security.Cryptography.X509Certificates;
 using System.Threading;
 using System.Collections.Generic;
 using Twitterizer;
+using Twitterizer.Core;
 using Smuxi.Common;
 
 namespace Smuxi.Engine
@@ -102,10 +103,8 @@ namespace Smuxi.Engine
         protected bool HasTokens {
             get {
                 return f_OAuthTokens != null &&
-                       !String.IsNullOrEmpty(f_OAuthTokens.ConsumerKey) &&
-                       !String.IsNullOrEmpty(f_OAuthTokens.ConsumerSecret) &&
-                       !String.IsNullOrEmpty(f_OAuthTokens.AccessToken) &&
-                       !String.IsNullOrEmpty(f_OAuthTokens.AccessTokenSecret);
+                       f_OAuthTokens.HasConsumerToken &&
+                       f_OAuthTokens.HasAccessToken;
             }
         }
 
@@ -125,6 +124,10 @@ namespace Smuxi.Engine
                 _("Home Timeline"),
                 this
             );
+            f_FriendsTimelineChat.InitMessageBuffer(
+                MessageBufferPersistencyType.Volatile
+            );
+            f_FriendsTimelineChat.ApplyConfig(Session.UserConfig);
             f_GroupChats.Add(f_FriendsTimelineChat);
 
             f_RepliesChat = new GroupChatModel(
@@ -132,6 +135,10 @@ namespace Smuxi.Engine
                 _("Replies"),
                 this
             );
+            f_RepliesChat.InitMessageBuffer(
+                MessageBufferPersistencyType.Volatile
+            );
+            f_RepliesChat.ApplyConfig(Session.UserConfig);
             f_GroupChats.Add(f_RepliesChat);
 
             f_DirectMessagesChat = new GroupChatModel(
@@ -139,34 +146,42 @@ namespace Smuxi.Engine
                 _("Direct Messages"),
                 this
             );
+            f_DirectMessagesChat.InitMessageBuffer(
+                MessageBufferPersistencyType.Volatile
+            );
+            f_DirectMessagesChat.ApplyConfig(Session.UserConfig);
             f_GroupChats.Add(f_DirectMessagesChat);
         }
 
-        public override void Connect(FrontendManager fm, string host, int port,
-                                     string username, string password)
+        public override void Connect(FrontendManager fm, ServerModel server)
         {
-            Trace.Call(fm, host, port, username, "XXX");
+            Trace.Call(fm, server);
 
-            f_Username = username;
-
-            var proxyType = (string) Session.UserConfig["Connection/ProxyType"];
-            if (proxyType.ToLower() == "http") {
-                var uriBuilder = new UriBuilder();
-                uriBuilder.Scheme = "http";
-                uriBuilder.Host = (string) Session.UserConfig["Connection/ProxyHostname"];
-                uriBuilder.Port = (int) Session.UserConfig["Connection/ProxyPort"];
-                uriBuilder.UserName = (string) Session.UserConfig["Connection/ProxyUsername"];
-                uriBuilder.Password = (string) Session.UserConfig["Connection/ProxyPassword"];
-                var proxyUri = uriBuilder.ToString();
-                f_WebProxy = new WebProxy(proxyUri);
+            if (fm == null) {
+                throw new ArgumentNullException("fm");
+            }
+            if (server == null) {
+                throw new ArgumentNullException("server");
             }
 
-            f_OptionalProperties = new OptionalProperties();
-            if (f_WebProxy != null) {
-                f_OptionalProperties.Proxy = f_WebProxy;
-            }
+            f_Username = server.Username;
 
-            f_ProtocolChat = new ProtocolChatModel(NetworkID, "Twitter " + username, this);
+            var proxySettings = new ProxySettings();
+            proxySettings.ApplyConfig(Session.UserConfig);
+            var twitterUrl = new OptionalProperties().APIBaseAddress;
+            f_WebProxy = proxySettings.GetWebProxy(twitterUrl);
+            // HACK: Twitterizer will always use the system proxy if set to null
+            // so explicitely override this by setting an empty proxy
+            if (f_WebProxy == null) {
+                f_WebProxy = new WebProxy();
+            }
+            f_OptionalProperties = CreateOptions<OptionalProperties>();
+
+            f_ProtocolChat = new ProtocolChatModel(NetworkID, "Twitter " + f_Username, this);
+            f_ProtocolChat.InitMessageBuffer(
+                MessageBufferPersistencyType.Volatile
+            );
+            f_ProtocolChat.ApplyConfig(Session.UserConfig);
             Session.AddChat(f_ProtocolChat);
             Session.SyncChat(f_ProtocolChat);
 
@@ -181,18 +196,28 @@ namespace Smuxi.Engine
                 f_OAuthTokens.ConsumerSecret = key[1];
 
                 MessageBuilder builder;
-                var servers = new ServerListController(Session.UserConfig);
-                var server = servers.GetServer(Protocol, username);
-                if (server != null) {
-                    if (password == null) {
-                        // no password passed, use server password
-                        password = server.Password;
+                var password = server.Password ?? String.Empty;
+                var access = password.Split('|');
+                if (access.Length == 2) {
+                    f_OAuthTokens.AccessToken = access[0];
+                    f_OAuthTokens.AccessTokenSecret = access[1];
+
+                    // verify access token
+                    var options = CreateOptions<VerifyCredentialsOptions>();
+                    var response = TwitterAccount.VerifyCredentials(
+                        f_OAuthTokens, options
+                    );
+                    if (response.Result == RequestResult.Unauthorized) {
+#if LOG4NET
+                        f_Logger.Warn("Connect(): Invalid access token, " +
+                                      "re-authorization required");
+#endif
+                        f_OAuthTokens.AccessToken = null;
+                        f_OAuthTokens.AccessTokenSecret = null;
                     }
                 }
 
-                password = password ?? String.Empty;
-                var access = password.Split('|');
-                if (password.Length == 0 || access.Length == 1) {
+                if (!f_OAuthTokens.HasAccessToken) {
                     // new account or basic auth user that needs to be migrated
                     var reqToken = OAuthUtility.GetRequestToken(key[0], key[1],
                                                             "oob", f_WebProxy);
@@ -202,14 +227,6 @@ namespace Smuxi.Engine
                     builder.AppendEventPrefix();
                     builder.AppendText(_("Twitter authorization required."));
                     Session.AddMessageToChat(f_ProtocolChat, builder.ToMessage());
-                    
-                    /*
-                        _("Twitter authorization required, please open the " +
-                          "following URL and enter the returned PIN using the " +
-                          "/pin command: {0}"),
-                        String.Empty
-                    );
-                    */
 
                     builder = CreateMessageBuilder();
                     builder.AppendEventPrefix();
@@ -240,9 +257,6 @@ namespace Smuxi.Engine
                     builder.AppendEventPrefix();
                     builder.AppendText(_("Please type: /pin PIN_FROM_TWITTER"));
                     Session.AddMessageToChat(f_ProtocolChat, builder.ToMessage());
-                } else {
-                    f_OAuthTokens.AccessToken = access[0];
-                    f_OAuthTokens.AccessTokenSecret = access[1];
                 }
             } catch (Exception ex) {
 #if LOG4NET
@@ -472,8 +486,10 @@ namespace Smuxi.Engine
                 return chat;
             }
 
-            TwitterUser user = TwitterUser.Show(f_OAuthTokens, userId,
-                                                f_OptionalProperties);
+            var response = TwitterUser.Show(f_OAuthTokens, userId,
+                                            f_OptionalProperties);
+            CheckResponse(response);
+            var user = response.ResponseObject;
             PersonModel person = CreatePerson(user);
             PersonChatModel personChat = new PersonChatModel(
                 person,
@@ -481,6 +497,10 @@ namespace Smuxi.Engine
                 user.ScreenName,
                 this
             );
+            personChat.InitMessageBuffer(
+                MessageBufferPersistencyType.Volatile
+            );
+            personChat.ApplyConfig(Session.UserConfig);
             Session.AddChat(personChat);
             Session.SyncChat(personChat);
             return personChat;
@@ -579,17 +599,11 @@ namespace Smuxi.Engine
 
         public void CommandHelp(CommandModel cd)
         {
-            MessageModel fmsg = new MessageModel();
-            TextMessagePartModel fmsgti;
-
-            fmsgti = new TextMessagePartModel();
+            var builder = CreateMessageBuilder();
             // TRANSLATOR: this line is used as a label / category for a
             // list of commands below
-            fmsgti.Text = "[" + _("Twitter Commands") + "]";
-            fmsgti.Bold = true;
-            fmsg.MessageParts.Add(fmsgti);
-
-            Session.AddMessageToChat(cd.Chat, fmsg);
+            builder.AppendHeader(_("Twitter Commands"));
+            cd.FrontendManager.AddMessageToChat(cd.Chat, builder.ToMessage());
 
             string[] help = {
                 "help",
@@ -598,32 +612,33 @@ namespace Smuxi.Engine
             };
 
             foreach (string line in help) {
-                cd.FrontendManager.AddTextToChat(cd.Chat, "-!- " + line);
+                builder = CreateMessageBuilder();
+                builder.AppendEventPrefix();
+                builder.AppendText(line);
+                cd.FrontendManager.AddMessageToChat(cd.Chat, builder.ToMessage());
             }
         }
 
         public void CommandConnect(CommandModel cd)
         {
-            string user;
+            var server = new ServerModel();
             if (cd.DataArray.Length >= 3) {
-                user = cd.DataArray[2];
+                server.Username = cd.DataArray[2];
             } else {
                 NotEnoughParameters(cd);
                 return;
             }
 
-            Connect(cd.FrontendManager, null, 0, user, null);
+            Connect(cd.FrontendManager, server);
         }
 
         public void CommandPin(CommandModel cd)
         {
-            string pin;
-            if (cd.DataArray.Length >= 2) {
-                pin = cd.DataArray[1];
-            } else {
+            if (String.IsNullOrEmpty(cd.Parameter)) {
                 NotEnoughParameters(cd);
                 return;
             }
+            var pin = cd.Parameter.Trim();
 
             MessageBuilder builder;
             if (String.IsNullOrEmpty(f_RequestToken)) {
@@ -670,6 +685,7 @@ namespace Smuxi.Engine
                       "the Twitter \"{0}\" account."),
                     f_Username
                 );
+                Session.AddMessageToChat(f_ProtocolChat, builder.ToMessage());
 
                 // allow the user to re-enter the pin
                 // LAME: An incorrect PIN invalidates the request token!
@@ -714,6 +730,7 @@ namespace Smuxi.Engine
                 // update token
                 server.Password = String.Format("{0}|{1}", response.Token,
                                                 response.TokenSecret);
+                servers.SetServer(server);
             }
             servers.Save();
 
@@ -783,15 +800,15 @@ namespace Smuxi.Engine
                 return;
             }
 
-            TwitterUser user = TwitterUser.Show(f_OAuthTokens, nickname,
-                                                f_OptionalProperties);
-            if (user.RequestStatus.Status != RequestResult.Success) {
+            var response = TwitterUser.Show(f_OAuthTokens, nickname,
+                                            f_OptionalProperties);
+            if (response.Result != RequestResult.Success) {
                 fm.AddTextToChat(cmd.Chat, "-!- " +
                     _("Could not send message - the specified user does not exist.")
                 );
                 return;
             }
-
+            var user = response.ResponseObject;
             var chat = OpenPrivateChat(user.Id);
 
             if (cmd.DataArray.Length >= 3) {
@@ -898,18 +915,22 @@ namespace Smuxi.Engine
 #if LOG4NET
             f_Logger.Debug("UpdateFriendsTimeline(): getting friend timeline from twitter...");
 #endif
-            var options = new TimelineOptions() {
-                Proxy = f_WebProxy,
-                SinceStatusId = f_LastFriendsTimelineStatusID,
-                Count = 50
-            };
-            var timeline = TwitterTimeline.HomeTimeline(f_OAuthTokens,
+            var options = CreateOptions<TimelineOptions>();
+            options.SinceStatusId = f_LastFriendsTimelineStatusID;
+            options.Count = 50;
+            var response = TwitterTimeline.HomeTimeline(f_OAuthTokens,
                                                         options);
+            // ignore temporarily issues
+            if (IsTemporilyErrorResponse(response)) {
+                return;
+            }
+            CheckResponse(response);
+            var timeline = response.ResponseObject;
 #if LOG4NET
             f_Logger.Debug("UpdateFriendsTimeline(): done. New tweets: " +
-                (timeline == null ? 0 : timeline.Count));
+                           timeline.Count);
 #endif
-            if (timeline == null || timeline.Count == 0) {
+            if (timeline.Count == 0) {
                 return;
             }
 
@@ -999,16 +1020,19 @@ namespace Smuxi.Engine
 #if LOG4NET
             f_Logger.Debug("UpdateReplies(): getting replies from twitter...");
 #endif
-            var options = new TimelineOptions() {
-                Proxy = f_WebProxy,
-                SinceStatusId = f_LastReplyStatusID
-            };
-            var timeline = TwitterTimeline.Mentions(f_OAuthTokens, options);
+            var options = CreateOptions<TimelineOptions>();
+            options.SinceStatusId = f_LastReplyStatusID;
+            var response = TwitterTimeline.Mentions(f_OAuthTokens, options);
+            // ignore temporarily issues
+            if (IsTemporilyErrorResponse(response)) {
+                return;
+            }
+            CheckResponse(response);
+            var timeline = response.ResponseObject;
 #if LOG4NET
-            f_Logger.Debug("UpdateReplies(): done. New replies: " +
-                (timeline == null ? 0 : timeline.Count));
+            f_Logger.Debug("UpdateReplies(): done. New replies: " + timeline.Count);
 #endif
-            if (timeline == null || timeline.Count == 0) {
+            if (timeline.Count == 0) {
                 return;
             }
 
@@ -1092,14 +1116,18 @@ namespace Smuxi.Engine
 #if LOG4NET
             f_Logger.Debug("UpdateDirectMessages(): getting received direct messages from twitter...");
 #endif
-            var options = new DirectMessagesOptions() {
-                Proxy = f_WebProxy,
-                SinceStatusId = f_LastDirectMessageReceivedStatusID,
-                Count = 50
-            };
-            var receivedTimeline = TwitterDirectMessage.DirectMessages(
+            var options = CreateOptions<DirectMessagesOptions>();
+            options.SinceStatusId = f_LastDirectMessageReceivedStatusID;
+            options.Count = 50;
+            var response = TwitterDirectMessage.DirectMessages(
                 f_OAuthTokens, options
             );
+            // ignore temporarily issues
+            if (IsTemporilyErrorResponse(response)) {
+                return;
+            }
+            CheckResponse(response);
+            var receivedTimeline = response.ResponseObject;
 #if LOG4NET
             f_Logger.Debug("UpdateDirectMessages(): done. New messages: " +
                 (receivedTimeline == null ? 0 : receivedTimeline.Count));
@@ -1108,14 +1136,18 @@ namespace Smuxi.Engine
 #if LOG4NET
             f_Logger.Debug("UpdateDirectMessages(): getting sent direct messages from twitter...");
 #endif
-            var sentOptions = new DirectMessagesSentOptions() {
-                Proxy = f_WebProxy,
-                SinceStatusId = f_LastDirectMessageSentStatusID,
-                Count = 50
-            };
-            var sentTimeline = TwitterDirectMessage.DirectMessagesSent(
+            var sentOptions = CreateOptions<DirectMessagesSentOptions>();
+            sentOptions.SinceStatusId = f_LastDirectMessageSentStatusID;
+            sentOptions.Count = 50;
+            response = TwitterDirectMessage.DirectMessagesSent(
                 f_OAuthTokens, sentOptions
             );
+            // ignore temporarily issues
+            if (IsTemporilyErrorResponse(response)) {
+                return;
+            }
+            CheckResponse(response);
+            var sentTimeline = response.ResponseObject;
 #if LOG4NET
             f_Logger.Debug("UpdateDirectMessages(): done. New messages: " +
                 (sentTimeline == null ? 0 : sentTimeline.Count));
@@ -1196,26 +1228,46 @@ namespace Smuxi.Engine
             }
 
 #if LOG4NET
-            f_Logger.Debug("UpdateFriends(): getting friends from twitter...");
+            f_Logger.Debug("UpdateFriends(): fetching friend IDs from twitter...");
 #endif
-            var options = new FriendsOptions() {
-                Proxy = f_WebProxy
-            };
-            TwitterUserCollection friends = TwitterFriendship.Friends(
+            var options = CreateOptions<UsersIdsOptions>();
+            options.UserId = f_TwitterUser.Id;
+            var response = TwitterFriendship.FriendsIds(
                 f_OAuthTokens, options
             );
+            CheckResponse(response);
+            var friendIds = response.ResponseObject;
 #if LOG4NET
-            f_Logger.Debug("UpdateFriends(): done. Friends: " +
-                (friends == null ? 0 : friends.Count));
+            f_Logger.Debug("UpdateFriends(): done. Fetched IDs: " + friendIds.Count);
 #endif
-            if (friends == null || friends.Count == 0) {
-                return;
-            }
 
-            var persons = new Dictionary<string, PersonModel>(friends.Count);
-            foreach (TwitterUser friend in friends) {
-                var person = CreatePerson(friend);
-                persons.Add(person.ID, person);
+            var persons = new Dictionary<string, PersonModel>(friendIds.Count);
+            // users/lookup only permits 100 users per call
+            var pageSize = 100;
+            var idList = new List<decimal>(friendIds);
+            var idPages = new List<List<decimal>>();
+            for (int offset = 0; offset < idList.Count; offset += pageSize) {
+                var count = Math.Min(pageSize, idList.Count - offset);
+                idPages.Add(idList.GetRange(offset, count));
+            }
+            foreach (var idPage in idPages) {
+#if LOG4NET
+                f_Logger.Debug("UpdateFriends(): fetching friends from twitter...");
+#endif
+                var userIds = new TwitterIdCollection(idPage);
+                var lookupOptions = CreateOptions<LookupUsersOptions>();
+                lookupOptions.UserIds = userIds;
+                lookupOptions.CacheOutput = true;
+                var lookupResponse = TwitterUser.Lookup(f_OAuthTokens, lookupOptions);
+                CheckResponse(lookupResponse);
+                var friends = lookupResponse.ResponseObject;
+#if LOG4NET
+                f_Logger.Debug("UpdateFriends(): done. Fetched friends: " + friends.Count);
+#endif
+                foreach (var friend in friends) {
+                    var person = CreatePerson(friend);
+                    persons.Add(person.ID, person);
+                }
             }
             f_Friends = persons;
         }
@@ -1225,11 +1277,10 @@ namespace Smuxi.Engine
 #if LOG4NET
             f_Logger.Debug("UpdateUser(): getting user details from twitter...");
 #endif
-            var user = TwitterUser.Show(f_OAuthTokens, f_Username,
-                                        f_OptionalProperties);
-            if (user == null || user.RequestStatus.Status != RequestResult.Success) {
-                throw new ApplicationException(user.RequestStatus.ErrorDetails.ErrorMessage);
-            }
+            var response = TwitterUser.Show(f_OAuthTokens, f_Username,
+                                            f_OptionalProperties);
+            CheckResponse(response);
+            var user = response.ResponseObject;
             f_TwitterUser = user;
 #if LOG4NET
             f_Logger.Debug("UpdateUser(): done.");
@@ -1259,17 +1310,19 @@ namespace Smuxi.Engine
             return builder.ToMessage();
         }
 
-        private void PostUpdate(string text)
+        private T CreateOptions<T>() where T : OptionalProperties, new()
         {
-            var options = new StatusUpdateOptions() {
+            var options = new T() {
                 Proxy = f_WebProxy
             };
+            return options;
+        }
+
+        private void PostUpdate(string text)
+        {
+            var options = CreateOptions<StatusUpdateOptions>();
             var res = TwitterStatus.Update(f_OAuthTokens, text, options);
-            if (res.RequestStatus.Status != RequestResult.Success) {
-                throw new ApplicationException(
-                    res.RequestStatus.ErrorDetails.ErrorMessage
-                );
-            }
+            CheckResponse(res);
             f_FriendsTimelineEvent.Set();
         }
 
@@ -1277,11 +1330,7 @@ namespace Smuxi.Engine
         {
             var res = TwitterDirectMessage.Send(f_OAuthTokens, target, text,
                                                 f_OptionalProperties);
-            if (res.RequestStatus.Status != RequestResult.Success) {
-                throw new ApplicationException(
-                    res.RequestStatus.ErrorDetails.ErrorMessage
-                );
-            }
+            CheckResponse(res);
             f_DirectMessageEvent.Set();
         }
         
@@ -1289,12 +1338,7 @@ namespace Smuxi.Engine
         {
             Trace.Call(exception == null ? null : exception.GetType());
 
-            /*
-            if (exception.RequestData != null &&
-                exception.RequestData.ResponseException != null) {
-                CheckWebException(exception.RequestData.ResponseException);
-                return;
-            } else */ if (exception.InnerException is WebException) {
+            if (exception.InnerException is WebException) {
                 CheckWebException((WebException) exception.InnerException);
                 return;
             } else if (exception.InnerException != null) {
@@ -1365,10 +1409,77 @@ namespace Smuxi.Engine
 #endif
                     return;
                 default:
+#if LOG4NET
+                    f_Logger.Error("CheckWebException(): " +
+                                   "Status: " + exception.Status + " " +
+                                   "ResponseUri: " + exception.Response.ResponseUri);
+#endif
                     throw exception;
             }
         }
 
+        private void CheckResponse<T>(TwitterResponse<T> response) where T : ITwitterObject
+        {
+            if (response == null) {
+                throw new ArgumentNullException("response");
+            }
+
+            if (response.Result == RequestResult.Success) {
+                return;
+            }
+
+#if LOG4NET
+            f_Logger.Error("CheckResponse(): " +
+                           "RequestUrl: " + response.RequestUrl + " " +
+                           "Result: " + response.Result + " " +
+                           "Content:\n" + response.Content);
+#endif
+
+            // HACK: Twitter returns HTML code saying they are overloaded o_O
+            if (response.Result == RequestResult.Unknown &&
+                response.ErrorMessage == null) {
+                response.ErrorMessage = _("Twitter didn't send a valid response, they're probably overloaded");
+            }
+            throw new TwitterizerException(response.ErrorMessage);
+        }
+
+        private bool IsTemporilyErrorResponse<T>(TwitterResponse<T> response)
+                                        where T : ITwitterObject
+        {
+            if (response == null) {
+                throw new ArgumentNullException("response");
+            }
+
+            switch (response.Result) {
+                case RequestResult.Success:
+                    // no error at all
+                    return false;
+                case RequestResult.ConnectionFailure:
+                case RequestResult.RateLimited:
+                case RequestResult.TwitterIsDown:
+                case RequestResult.TwitterIsOverloaded:
+                // probably "Twitter is over capacity"
+                case RequestResult.Unknown:
+#if LOG4NET
+                    f_Logger.Debug("IsTemporilyErrorResponse(): " +
+                                   "Detected temporily error " +
+                                   "RequestUrl: " + response.RequestUrl + " " +
+                                   "Result: " + response.Result + " " +
+                                   "Content:\n" + response.Content);
+#endif
+                    return true;
+            }
+
+#if LOG4NET
+            f_Logger.Debug("IsTemporilyErrorResponse(): " +
+                           "Detected permanent error " +
+                           "RequestUrl: " + response.RequestUrl + " " +
+                           "Result: " + response.Result + " " +
+                           "Content:\n" + response.Content);
+#endif
+            return false;
+        }
+
         private PersonModel GetPerson(TwitterUser user)
         {
             if (user == null) {
@@ -1382,6 +1493,14 @@ namespace Smuxi.Engine
             return person;
         }
 
+        private PersonModel CreatePerson(decimal userId)
+        {
+            var res = TwitterUser.Show(f_OAuthTokens, userId, f_OptionalProperties);
+            CheckResponse(res);
+            var user = res.ResponseObject;
+            return CreatePerson(user);
+        }
+
         private PersonModel CreatePerson(TwitterUser user)
         {
             if (user == null) {
@@ -1424,8 +1543,7 @@ namespace Smuxi.Engine
             if (sslPolicyErrors == SslPolicyErrors.RemoteCertificateChainErrors &&
                 sender is HttpWebRequest) {
                 var request = (HttpWebRequest) sender;
-                if (request.RequestUri.Host == "api.twitter.com" &&
-                    certificate.Issuer == "OU=Equifax Secure Certificate Authority, O=Equifax, C=US") {
+                if (request.RequestUri.Host == "api.twitter.com") {
                     return true;
                 }
             }
