@@ -40,6 +40,7 @@ using jabber.protocol;
 using jabber.protocol.client;
 using jabber.protocol.iq;
 using XmppMessageType = jabber.protocol.client.MessageType;
+using XmppProxyType = jabber.connection.ProxyType;
 
 using Smuxi.Common;
 
@@ -260,6 +261,10 @@ namespace Smuxi.Engine
                             CommandPart(command);
                             handled = true;
                             break;
+                        case "away":
+                            CommandAway(command);
+                            handled = true;
+                            break;
                     }
                 } else {
                     _Say(command.Chat, command.Data);
@@ -305,6 +310,11 @@ namespace Smuxi.Engine
             string[] help = {
             "help",
             "connect xmpp/jabber server port username password [resource]",
+            "msg/query jid message",
+            "say message",
+            "join muc-jid",
+            "part/leave [muc-jid]",
+            "away [away-message]"
             };
             
             foreach (string line in help) { 
@@ -421,6 +431,15 @@ namespace Smuxi.Engine
             }
         }
 
+        public void CommandAway(CommandModel cd)
+        {
+            if (cd.DataArray.Length >= 2) {
+                SetPresenceStatus(PresenceStatus.Away, cd.Parameter);
+            } else {
+                SetPresenceStatus(PresenceStatus.Online, null);
+            }
+        }
+
         public void CommandSay(CommandModel cd)
         {
             _Say(cd.Chat, cd.Parameter);
@@ -806,6 +825,35 @@ namespace Smuxi.Engine
             if (!server.ValidateServerCertificate) {
                 _JabberClient.OnInvalidCertificate += ValidateCertificate;
             }
+
+            var proxySettings = new ProxySettings();
+            proxySettings.ApplyConfig(Session.UserConfig);
+            var protocol = server.UseEncryption ? "xmpps" : "xmpp";
+            var serverUri = String.Format("{0}://{1}:{2}", protocol,
+                                          server.Hostname, server.Port);
+            var proxy = proxySettings.GetWebProxy(serverUri);
+            if (proxy == null) {
+                _JabberClient.Proxy = XmppProxyType.None;
+            } else {
+                var proxyScheme = proxy.Address.Scheme;
+                var xmppProxyType = XmppProxyType.None;
+                try {
+                    // HACK: map proxy scheme to SmartIrc4net's ProxyType
+                    xmppProxyType = (XmppProxyType) Enum.Parse(
+                        typeof(XmppProxyType), proxyScheme, true
+                    );
+                } catch (ArgumentException ex) {
+#if LOG4NET
+                    _Logger.Error("ApplyConfig(): Couldn't parse proxy type: " +
+                                  proxyScheme, ex);
+#endif
+                }
+                _JabberClient.Proxy = xmppProxyType;
+                _JabberClient.ProxyHost = proxy.Address.Host;
+                _JabberClient.ProxyPort = proxy.Address.Port;
+                _JabberClient.ProxyUsername = proxySettings.ProxyUsername;
+                _JabberClient.ProxyPassword = proxySettings.ProxyPassword;
+            }
         }
 
         private static bool ValidateCertificate(object sender,
