@@ -37,6 +37,7 @@ namespace Smuxi.Engine
 #if LOG4NET
         private static readonly log4net.ILog _Logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);
 #endif
+        private static readonly string _LibraryTextDomain = "smuxi-engine";
         private UserConfig _UserConfig;
         
         public ServerListController(UserConfig userConfig)
@@ -95,6 +96,9 @@ namespace Smuxi.Engine
             server.Network     = (string) _UserConfig[prefix + "Network"];
             server.Username    = (string) _UserConfig[prefix + "Username"];
             server.Password    = (string) _UserConfig[prefix + "Password"];
+            server.UseEncryption = (bool) _UserConfig[prefix + "UseEncryption"];
+            server.ValidateServerCertificate =
+                (bool) _UserConfig[prefix + "ValidateServerCertificate"];
             if (_UserConfig[prefix + "OnStartupConnect"] != null) {
                 server.OnStartupConnect = (bool) _UserConfig[prefix + "OnStartupConnect"];
             }
@@ -123,10 +127,19 @@ namespace Smuxi.Engine
             if (server == null) {
                 throw new ArgumentNullException("server");
             }
+            if (String.IsNullOrEmpty(server.Hostname)) {
+                throw new InvalidOperationException(_("Server hostname must not be empty."));
+            }
+            if (server.Hostname.Contains("\n")) {
+                throw new InvalidOperationException(_("Server hostname contains invalid characters (newline)."));
+            }
             foreach (var s in GetServerList()) {
                 if (s.Protocol == server.Protocol &&
                     s.Hostname == server.Hostname) {
-                    throw new InvalidOperationException("Server '" + server.Hostname + "' already exists.");
+                    throw new InvalidOperationException(
+                        String.Format(_("Server '{0}' already exists."),
+                                      server.Hostname)
+                    );
                 }
             }
 
@@ -136,6 +149,9 @@ namespace Smuxi.Engine
             _UserConfig[prefix + "Network"]  = server.Network;
             _UserConfig[prefix + "Username"] = server.Username;
             _UserConfig[prefix + "Password"] = server.Password;
+            _UserConfig[prefix + "UseEncryption"] = server.UseEncryption;
+            _UserConfig[prefix + "ValidateServerCertificate"] =
+                server.ValidateServerCertificate;
             _UserConfig[prefix + "OnStartupConnect"] = server.OnStartupConnect;
             _UserConfig[prefix + "OnConnectCommands"] = server.OnConnectCommands;
             
@@ -162,6 +178,9 @@ namespace Smuxi.Engine
             _UserConfig[prefix + "Network"]  = server.Network;
             _UserConfig[prefix + "Username"] = server.Username;
             _UserConfig[prefix + "Password"] = server.Password;
+            _UserConfig[prefix + "UseEncryption"] = server.UseEncryption;
+            _UserConfig[prefix + "ValidateServerCertificate"] =
+                server.ValidateServerCertificate;
             _UserConfig[prefix + "OnStartupConnect"] = server.OnStartupConnect;
             _UserConfig[prefix + "OnConnectCommands"] = server.OnConnectCommands;
         }
@@ -194,5 +213,10 @@ namespace Smuxi.Engine
         {
             _UserConfig.Save();
         }
+
+        private static string _(string msg)
+        {
+            return LibraryCatalog.GetString(msg, _LibraryTextDomain);
+        }
     }
 }
