@@ -1,13 +1,7 @@
 /*
- * $Id$
- * $URL$
- * $Rev$
- * $Author$
- * $Date$
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2008 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2011 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -51,7 +45,9 @@ namespace Smuxi.Engine
         private bool             _IsFrontendSynced;
         private IList<ChatModel> _SyncedChats = new List<ChatModel>();
         private TaskQueue        f_TaskQueue;
-        
+
+        DateTime LastConfigChange;
+
         public int Version {
             get {
                 return 0;
@@ -113,10 +109,9 @@ namespace Smuxi.Engine
             f_TaskQueue = new TaskQueue("FrontendManager");
             f_TaskQueue.ExceptionEvent += OnTaskQueueExceptionEvent;
             f_TaskQueue.AbortedEvent   += OnTaskQueueAbortedEvent;
-            
+
             // register event for config invalidation
-            // BUG: when the frontend disconnects there are dangling methods registered!
-            //_Session.Config.Changed += new EventHandler(_OnConfigChanged);
+            _Session.Config.Changed += _OnConfigChanged;
         }
         
         ~FrontendManager()
@@ -140,6 +135,7 @@ namespace Smuxi.Engine
             
             if (disposing) {
                 f_TaskQueue.Dispose();
+                _Session.Config.Changed -= _OnConfigChanged;
             }
         }
         
@@ -182,7 +178,10 @@ namespace Smuxi.Engine
 
             _Session.CheckPresenceStatus();
         }
-        
+
+        /// <remarks>
+        /// This method is thread safe.
+        /// </remarks>
         public void AddSyncedChat(ChatModel chatModel)
         {
             Trace.Call(chatModel);
@@ -197,7 +196,11 @@ namespace Smuxi.Engine
                 return;
             }
 
-            _SyncedChats.Add(chatModel);
+            // this method must be thread-safe as the frontend might sync
+            // multiple chats at the same time
+            lock (_SyncedChats) {
+                _SyncedChats.Add(chatModel);
+            }
         }
         
         public void NextProtocolManager()
@@ -233,7 +236,7 @@ namespace Smuxi.Engine
         
         public void AddChat(ChatModel chat)
         {
-            if (!_SyncedChats.Contains(chat) && _IsFrontendSynced) {
+            if (!IsSynced(chat) && _IsFrontendSynced) {
                 _AddChat(chat);
             }
         }
@@ -265,7 +268,7 @@ namespace Smuxi.Engine
         
         public void DisableChat(ChatModel chat)
         {
-            if (_SyncedChats.Contains(chat)) {
+            lock (_SyncedChats) {
                 _SyncedChats.Remove(chat);
             }
             f_TaskQueue.Queue(delegate {
@@ -275,7 +278,7 @@ namespace Smuxi.Engine
         
         public void AddMessageToChat(ChatModel chat, MessageModel msg)
         {
-            if (!_SyncedChats.Contains(chat)) {
+            if (!IsSynced(chat)) {
 #if LOG4NET
                 // too much logging noise
                 //_Logger.Warn("AddMessageToChat(): chat: " + chat + " is not synced yet, ignoring call...");
@@ -302,7 +305,7 @@ namespace Smuxi.Engine
         
         public void RemoveChat(ChatModel chat)
         {
-            if (_SyncedChats.Contains(chat)) {
+            lock (_SyncedChats) {
                 _SyncedChats.Remove(chat);
             }
             f_TaskQueue.Queue(delegate {
@@ -312,7 +315,7 @@ namespace Smuxi.Engine
         
         public void SyncChat(ChatModel chat)
         {
-            if (!_SyncedChats.Contains(chat) && _IsFrontendSynced) {
+            if (!IsSynced(chat) && _IsFrontendSynced) {
                 _SyncChat(chat);
             }
         }
@@ -326,7 +329,7 @@ namespace Smuxi.Engine
         
         public void AddPersonToGroupChat(GroupChatModel groupChat, PersonModel person)
         {
-            if (!_SyncedChats.Contains(groupChat)) {
+            if (!IsSynced(groupChat)) {
                 return;
             }
             
@@ -342,7 +345,7 @@ namespace Smuxi.Engine
         
         public void UpdatePersonInGroupChat(GroupChatModel groupChat, PersonModel oldPerson, PersonModel newPerson)
         {
-            if (!_SyncedChats.Contains(groupChat)) {
+            if (!IsSynced(groupChat)) {
                 return;
             }
             
@@ -358,7 +361,7 @@ namespace Smuxi.Engine
     
         public void UpdateTopicInGroupChat(GroupChatModel groupChat, MessageModel topic)
         {
-            if (!_SyncedChats.Contains(groupChat)) {
+            if (!IsSynced(groupChat)) {
                 return;
             }
             
@@ -374,7 +377,7 @@ namespace Smuxi.Engine
     
         public void RemovePersonFromGroupChat(GroupChatModel groupChat, PersonModel person)
         {
-            if (!_SyncedChats.Contains(groupChat)) {
+            if (!IsSynced(groupChat)) {
                 return;
             }
             
@@ -405,21 +408,42 @@ namespace Smuxi.Engine
         private void _OnConfigChanged(object sender, EventArgs e)
         {
             Trace.Call(sender, e);
-            
-            // BUG: we should use some timeout here and only call the delegate
-            // when the timeout is reached, else we flood the frontend for each
-            // changed value in the config!
+
+            // only push config changes once per 30 seconds
+            if ((DateTime.UtcNow - LastConfigChange).TotalSeconds < 30) {
+                return;
+            }
+
             try {
+                // DISABLED: delegate is not reliable enough, this needs to be
+                // replaced with an IChatConfig API
+                /*
                 if (_ConfigChangedDelegate != null) {
                     _ConfigChangedDelegate();
                 }
+                */
             } catch (Exception ex) {
 #if LOG4NET
                 _Logger.Error(ex);
 #endif
             }
+            LastConfigChange = DateTime.UtcNow;
         }
-        
+
+        /// <remarks>
+        /// This method is thread safe.
+        /// </remarks>
+        bool IsSynced(ChatModel chatModel)
+        {
+            if (chatModel == null) {
+                throw new ArgumentNullException("chatModel");
+            }
+
+            lock (_SyncedChats) {
+                return _SyncedChats.Contains(chatModel);
+            }
+        }
+
         protected virtual void OnTaskQueueExceptionEvent(object sender, TaskQueueExceptionEventArgs e)
         {
             Trace.Call(sender, e);
