@@ -1,13 +1,7 @@
 /*
- * $Id: GroupChatView.cs 188 2007-04-21 22:03:54Z meebey $
- * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Frontend-GNOME/GroupChatView.cs $
- * $Rev: 188 $
- * $Author: meebey $
- * $Date: 2007-04-22 00:03:54 +0200 (Sun, 22 Apr 2007) $
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2008 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2008, 2010-2011 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -27,6 +21,7 @@
  */
 
 using System;
+using System.Threading;
 using System.Globalization;
 using Smuxi.Engine;
 using Smuxi.Common;
@@ -43,11 +38,18 @@ namespace Smuxi.Frontend.Gnome
         {
             Trace.Call(personChat);
 
-            _IrcProtocolManager = (IrcProtocolManager)personChat.ProtocolManager;
-
             OutputMessageTextView.PopulatePopup += _OnOutputMessageTextViewPopulatePopup;
         }
-        
+
+        public override void Sync()
+        {
+            Trace.Call();
+
+            base.Sync();
+
+            _IrcProtocolManager = (IrcProtocolManager) ProtocolManager;
+        }
+
         private void _OnOutputMessageTextViewPopulatePopup (object o, Gtk.PopulatePopupArgs args)
         {
             if (OutputMessageTextView.IsAtUrlTag) {
@@ -83,13 +85,19 @@ namespace Smuxi.Frontend.Gnome
         {
             Trace.Call(sender, e);
 
-            _IrcProtocolManager.CommandWhoIs(
-                new CommandModel(
-                    Frontend.FrontendManager,
-                    ChatModel,
-                    ChatModel.ID
-                )
-             );
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    _IrcProtocolManager.CommandWhoIs(
+                        new CommandModel(
+                            Frontend.FrontendManager,
+                            ChatModel,
+                            ChatModel.ID
+                        )
+                     );
+                } catch (Exception ex) {
+                    Frontend.ShowException(ex);
+                }
+            });
         }
 
         private static string _(string msg)
