@@ -973,9 +973,7 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(sender, e);
             
             try {
-                if (_Notebook.CurrentPage < _Notebook.NPages) {
-                    _Notebook.CurrentPage++;
-                }
+                ChatViewManager.CurrentChatNumber++;
             } catch (Exception ex) {
                 Frontend.ShowException(this, ex);
             }
@@ -986,9 +984,7 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(sender, e);
             
             try {
-                if (_Notebook.CurrentPage > 0) {
-                    _Notebook.CurrentPage--;
-                }
+                ChatViewManager.CurrentChatNumber--;
             } catch (Exception ex) {
                 Frontend.ShowException(this, ex);
             }
