@@ -137,7 +137,7 @@ namespace Smuxi.Frontend.Gnome
             ((Gtk.CheckButton)_Glade["OverrideForegroundColorCheckButton"]).Toggled += OnOverrideForegroundColorCheckButtonToggled;
             ((Gtk.CheckButton)_Glade["OverrideBackgroundColorCheckButton"]).Toggled += OnOverrideBackgroundColorCheckButtonToggled;
             ((Gtk.CheckButton)_Glade["OverrideFontCheckButton"]).Toggled += OnOverrideFontCheckButtonToggled;
-
+            ((Gtk.FontButton)_Glade["FontButton"]).FontSet += _OnChanged;
             ((Gtk.CheckButton)_Glade["ShowAdvancedSettingsCheckButton"]).Toggled += delegate {
                 CheckShowAdvancedSettingsCheckButton();
             };
@@ -181,6 +181,30 @@ namespace Smuxi.Frontend.Gnome
             wrapModeComboBox.Model = store;
             wrapModeComboBox.Active = 0;
             
+            Gtk.ComboBox persistencyTypeComboBox =
+                (Gtk.ComboBox) _Glade["PersistencyTypeComboBox"];
+            // glade might initialize it already!
+            persistencyTypeComboBox.Clear();
+            persistencyTypeComboBox.Changed += _OnChanged;
+            cell = new Gtk.CellRendererText();
+            persistencyTypeComboBox.PackStart(cell, false);
+            persistencyTypeComboBox.AddAttribute(cell, "text", 1);
+            store = new Gtk.ListStore(
+                typeof(MessageBufferPersistencyType), typeof(string)
+            );
+            // fill ListStore
+            store.AppendValues(MessageBufferPersistencyType.Volatile,
+                               _("Volatile"));
+            store.AppendValues(MessageBufferPersistencyType.Persistent,
+                               _("Persistent"));
+            persistencyTypeComboBox.Model = store;
+            persistencyTypeComboBox.Active = 0;
+            if (Frontend.EngineVersion < new Version("0.8.1")) {
+                persistencyTypeComboBox.Sensitive = false;
+                ((Gtk.SpinButton) _Glade["VolatileMaxCapacitySpinButton"]).Sensitive = false;
+                ((Gtk.SpinButton) _Glade["PersistentMaxCapacitySpinButton"]).Sensitive = false;
+            }
+
             Gtk.ComboBox proxyTypeComboBox = (Gtk.ComboBox)_Glade["ProxyTypeComboBox"];
             // initialize wrap modes
             // glade might initialize it already!
@@ -196,6 +220,8 @@ namespace Smuxi.Frontend.Gnome
             // fill ListStore
             store.AppendValues(ProxyType.None,    String.Format("<{0}>",
                                                                 _("No Proxy")));
+            store.AppendValues(ProxyType.System,  String.Format("<{0}>",
+                                                                _("System Default")));
             store.AppendValues(ProxyType.Http,    "HTTP");
             store.AppendValues(ProxyType.Socks4,  "SOCK 4");
             store.AppendValues(ProxyType.Socks4a, "SOCK 4a");
@@ -285,7 +311,12 @@ namespace Smuxi.Frontend.Gnome
                 try {
                     Encoding enc = Encoding.GetEncoding(encInfo.CodePage);
                     string encodingName = enc.EncodingName.ToUpper();
-                    
+
+                    if (!enc.IsSingleByte && enc != Encoding.UTF8) {
+                        // ignore multi byte encodings except UTF-8
+                        continue;
+                    }
+
                     // filter noise and duplicates
                     if (encodingName.IndexOf("DOS") != -1 ||
                         encodingName.IndexOf("MAC") != -1 ||
@@ -350,6 +381,29 @@ namespace Smuxi.Frontend.Gnome
                 (string) Frontend.UserConfig["Connection/ProxyPassword"];
             CheckProxyShowPasswordCheckButton();
 
+            // MessageBuffer
+            if (Frontend.EngineVersion >= new Version("0.8.1")) {
+                // feature introduced in >= 0.8.1
+                Gtk.ComboBox persistencyTypeComboBox =
+                    ((Gtk.ComboBox)_Glade["PersistencyTypeComboBox"]);
+                var persistencyType = (MessageBufferPersistencyType) Enum.Parse(
+                    typeof(MessageBufferPersistencyType),
+                    (string) Frontend.UserConfig["MessageBuffer/PersistencyType"]
+                );
+                i = 0;
+                foreach (object[] row in (Gtk.ListStore) persistencyTypeComboBox.Model) {
+                    if (((MessageBufferPersistencyType) row[0]) == persistencyType) {
+                        persistencyTypeComboBox.Active = i;
+                        break;
+                    }
+                    i++;
+                }
+                ((Gtk.SpinButton)_Glade["VolatileMaxCapacitySpinButton"]).Value =
+                    (double)(int)Frontend.UserConfig["MessageBuffer/Volatile/MaxCapacity"];
+                ((Gtk.SpinButton)_Glade["PersistentMaxCapacitySpinButton"]).Value =
+                    (double)(int)Frontend.UserConfig["MessageBuffer/Persistent/MaxCapacity"];
+            }
+
             // Interface
             ((Gtk.CheckButton) _Glade["ShowAdvancedSettingsCheckButton"]).Active =
                 (bool) Frontend.UserConfig["Interface/ShowAdvancedSettings"];
@@ -416,19 +470,19 @@ namespace Smuxi.Frontend.Gnome
             
             colorButton = (Gtk.ColorButton)_Glade["NoActivityColorButton"];
             colorHexCode = (string)Frontend.UserConfig["Interface/Notebook/Tab/NoActivityColor"];
-            colorButton.Color = ColorTools.GetGdkColor(colorHexCode);
+            colorButton.Color = ColorConverter.GetGdkColor(colorHexCode);
 
             colorButton = (Gtk.ColorButton)_Glade["ActivityColorButton"];
             colorHexCode = (string)Frontend.UserConfig["Interface/Notebook/Tab/ActivityColor"];
-            colorButton.Color = ColorTools.GetGdkColor(colorHexCode);
+            colorButton.Color = ColorConverter.GetGdkColor(colorHexCode);
 
             colorButton = (Gtk.ColorButton)_Glade["ModeColorButton"];
             colorHexCode = (string)Frontend.UserConfig["Interface/Notebook/Tab/EventColor"];
-            colorButton.Color = ColorTools.GetGdkColor(colorHexCode);
+            colorButton.Color = ColorConverter.GetGdkColor(colorHexCode);
             
             colorButton = (Gtk.ColorButton)_Glade["HighlightColorButton"];
             colorHexCode = (string)Frontend.UserConfig["Interface/Notebook/Tab/HighlightColor"];
-            colorButton.Color = ColorTools.GetGdkColor(colorHexCode);
+            colorButton.Color = ColorConverter.GetGdkColor(colorHexCode);
             
             // Interface/Chat
             colorButton = (Gtk.ColorButton)_Glade["ForegroundColorButton"];
@@ -437,7 +491,7 @@ namespace Smuxi.Frontend.Gnome
                 ((Gtk.CheckButton)_Glade["OverrideForegroundColorCheckButton"]).Active = false;
             } else {
                 ((Gtk.CheckButton)_Glade["OverrideForegroundColorCheckButton"]).Active = true;
-                colorButton.Color = ColorTools.GetGdkColor(colorHexCode);
+                colorButton.Color = ColorConverter.GetGdkColor(colorHexCode);
             }
             
             colorButton = (Gtk.ColorButton)_Glade["BackgroundColorButton"];
@@ -446,7 +500,7 @@ namespace Smuxi.Frontend.Gnome
                 ((Gtk.CheckButton)_Glade["OverrideBackgroundColorCheckButton"]).Active = false;
             } else {
                 ((Gtk.CheckButton)_Glade["OverrideBackgroundColorCheckButton"]).Active = true;
-                colorButton.Color = ColorTools.GetGdkColor(colorHexCode);
+                colorButton.Color = ColorConverter.GetGdkColor(colorHexCode);
             }
             
             Gtk.FontButton fontButton = (Gtk.FontButton)_Glade["FontButton"];
@@ -624,6 +678,27 @@ namespace Smuxi.Frontend.Gnome
             Frontend.UserConfig["Connection/ProxyPassword"] =
                 ((Gtk.Entry) _Glade["ProxyPasswordEntry"]).Text;
 
+            int i;
+            // MessageBuffer
+            if (Frontend.EngineVersion >= new Version("0.8.1")) {
+                var persistencyTypeComboBox = (Gtk.ComboBox) _Glade["PersistencyTypeComboBox"];
+                var persistencyType = MessageBufferPersistencyType.Volatile;
+                i = 0;
+                foreach (object[] row in (Gtk.ListStore) persistencyTypeComboBox.Model) {
+                    if (persistencyTypeComboBox.Active == i) {
+                        persistencyType = (MessageBufferPersistencyType) row[0];
+                        break;
+                    }
+                    i++;
+                }
+                Frontend.UserConfig["MessageBuffer/PersistencyType"] =
+                    persistencyType.ToString();
+                Frontend.UserConfig["MessageBuffer/Volatile/MaxCapacity"] =
+                    (int)((Gtk.SpinButton)_Glade["VolatileMaxCapacitySpinButton"]).Value;
+                Frontend.UserConfig["MessageBuffer/Persistent/MaxCapacity"] =
+                    (int)((Gtk.SpinButton)_Glade["PersistentMaxCapacitySpinButton"]).Value;
+            }
+
             // Interface
             Frontend.UserConfig["Interface/ShowAdvancedSettings"] =
                 ((Gtk.CheckButton)_Glade["ShowAdvancedSettingsCheckButton"]).Active;
@@ -636,7 +711,7 @@ namespace Smuxi.Frontend.Gnome
                 (int)((Gtk.SpinButton)_Glade["EngineBufferLinesSpinButton"]).Value;
             Frontend.UserConfig["Interface/Notebook/StripColors"] =
                 ((Gtk.CheckButton)_Glade["StripColorsCheckButton"]).Active;
-            Frontend.UserConfig["Interface/Notebook/StripFormatting"] =
+            Frontend.UserConfig["Interface/Notebook/StripFormattings"] =
                 ((Gtk.CheckButton)_Glade["StripFormattingsCheckButton"]).Active;
                 
             string tab_position = null;
@@ -679,25 +754,25 @@ namespace Smuxi.Frontend.Gnome
             // Interface/Notebook/Tab
             prefix = "Interface/Notebook/Tab/";
             Frontend.UserConfig[prefix + "NoActivityColor"] =
-                ColorTools.GetHexCodeColor(((Gtk.ColorButton)_Glade["NoActivityColorButton"]).Color);
+                ColorConverter.GetHexCode(((Gtk.ColorButton)_Glade["NoActivityColorButton"]).Color);
             Frontend.UserConfig[prefix + "ActivityColor"] =
-                ColorTools.GetHexCodeColor(((Gtk.ColorButton)_Glade["ActivityColorButton"]).Color);
+                ColorConverter.GetHexCode(((Gtk.ColorButton)_Glade["ActivityColorButton"]).Color);
             Frontend.UserConfig[prefix + "EventColor"] =
-                ColorTools.GetHexCodeColor(((Gtk.ColorButton)_Glade["ModeColorButton"]).Color);
+                ColorConverter.GetHexCode(((Gtk.ColorButton)_Glade["ModeColorButton"]).Color);
             Frontend.UserConfig[prefix + "HighlightColor"] =
-                ColorTools.GetHexCodeColor(((Gtk.ColorButton)_Glade["HighlightColorButton"]).Color);
+                ColorConverter.GetHexCode(((Gtk.ColorButton)_Glade["HighlightColorButton"]).Color);
             
             // Interface/Chat
             prefix = "Interface/Chat/";
             if (((Gtk.CheckButton)_Glade["OverrideForegroundColorCheckButton"]).Active) {
                 Frontend.UserConfig[prefix + "ForegroundColor"] = 
-                    ColorTools.GetHexCodeColor(((Gtk.ColorButton)_Glade["ForegroundColorButton"]).Color);
+                    ColorConverter.GetHexCode(((Gtk.ColorButton)_Glade["ForegroundColorButton"]).Color);
             } else {
                 Frontend.UserConfig[prefix + "ForegroundColor"] = String.Empty;
             }
             if (((Gtk.CheckButton)_Glade["OverrideBackgroundColorCheckButton"]).Active) {
                 Frontend.UserConfig[prefix + "BackgroundColor"] = 
-                    ColorTools.GetHexCodeColor(((Gtk.ColorButton)_Glade["BackgroundColorButton"]).Color);
+                    ColorConverter.GetHexCode(((Gtk.ColorButton)_Glade["BackgroundColorButton"]).Color);
             } else {
                 Frontend.UserConfig[prefix + "BackgroundColor"] = String.Empty;
             }
@@ -715,7 +790,7 @@ namespace Smuxi.Frontend.Gnome
             
             Gtk.ComboBox wrapModeComboBox = (Gtk.ComboBox) _Glade["WrapModeComboBox"];
             Gtk.WrapMode wrapMode = Gtk.WrapMode.Char;
-            int i = 0;
+            i = 0;
             foreach (object[] row in (Gtk.ListStore) wrapModeComboBox.Model) {
                 if (wrapModeComboBox.Active == i) {
                     wrapMode = (Gtk.WrapMode) row[0];
@@ -807,7 +882,6 @@ namespace Smuxi.Frontend.Gnome
             try {
                 _Save();
                 Frontend.Config.Load();
-                Frontend.UserConfig.ClearCache();
                 Frontend.ApplyConfig(Frontend.UserConfig);
                 _Dialog.Destroy();
             } catch (ApplicationException ex) {
@@ -827,7 +901,6 @@ namespace Smuxi.Frontend.Gnome
             try {
                 _Save();
                 Frontend.Config.Load();
-                Frontend.UserConfig.ClearCache();
                 _Load();
                 Frontend.ApplyConfig(Frontend.UserConfig);
             } catch (ApplicationException ex) {
@@ -956,6 +1029,7 @@ namespace Smuxi.Frontend.Gnome
             var proxyType = (ProxyType) typoComboBox.Model.GetValue(iter, 0);
             switch (proxyType) {
                 case ProxyType.None:
+                case ProxyType.System:
                     hostEntry.Sensitive = false;
                     portSpinButton.Sensitive = false;
                     userEntry.Sensitive = false;
