@@ -39,8 +39,17 @@ namespace Smuxi.Frontend.Gnome
         public  PersonChatModel PersonChatModel { get; private set; }
         public  PersonModel     PersonModel { get; private set; }
 
-        static PersonChatView() {
-            IconPixbuf = new Gdk.Pixbuf(null, "person-chat.svg", 16, 16);
+        protected override Gtk.Image DefaultTabImage {
+            get {
+                return new Gtk.Image(IconPixbuf);
+            }
+        }
+
+        static PersonChatView()
+        {
+            IconPixbuf = Frontend.LoadIcon(
+                "smuxi-person-chat", 16, "person-chat_256x256.png"
+            );
         }
 
         public PersonChatView(PersonChatModel chat) : base(chat)
@@ -49,10 +58,6 @@ namespace Smuxi.Frontend.Gnome
             
             PersonChatModel = chat;
 
-            var tabImage = new Gtk.Image(IconPixbuf);
-            TabHBox.PackStart(tabImage, false, false, 2);
-            TabHBox.ShowAll();
-            
             Add(OutputScrolledWindow);
         }
 
@@ -60,6 +65,12 @@ namespace Smuxi.Frontend.Gnome
         {
             Trace.Call();
 
+            GLib.Idle.Add(delegate {
+                TabImage.SetFromStock(Gtk.Stock.Refresh, Gtk.IconSize.Menu);
+                return false;
+            });
+
+            // REMOTING CALL 1
             PersonModel = PersonChatModel.Person;
 
             base.Sync();
