@@ -1,13 +1,7 @@
 /*
- * $Id$
- * $URL$
- * $Rev$
- * $Author$
- * $Date$
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2005-2006 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2005-2006, 2009-2011 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -27,6 +21,7 @@
  */
 
 using System;
+using System.Threading;
 using Smuxi.Engine;
 using Smuxi.Common;
 
@@ -37,18 +32,23 @@ namespace Smuxi.Frontend.Gnome
     {
         public static Gdk.Pixbuf IconPixbuf { get; private set; }
         
-        static ProtocolChatView() {
-            IconPixbuf = new Gdk.Pixbuf(null, "protocol-chat.svg", 16, 16);
+        protected override Gtk.Image DefaultTabImage {
+            get {
+                return new Gtk.Image(IconPixbuf);
+            }
+        }
+
+        static ProtocolChatView()
+        {
+            IconPixbuf = Frontend.LoadIcon(
+                "smuxi-protocol-chat", 16, "protocol-chat_256x256.png"
+            );
         }
 
         public ProtocolChatView(ChatModel chat) : base(chat)
         {
             Trace.Call(chat);
             
-            var tabImage = new Gtk.Image(IconPixbuf);
-            TabHBox.PackStart(tabImage, false, false, 2);
-            TabHBox.ShowAll();
-            
             Add(OutputScrolledWindow);
             
             ShowAll();
@@ -59,6 +59,7 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call();
             
             // show warning if there are open chats (besides protocol chat)
+            // FIXME: REMOTING CALL 1 + 2 + 3
             if (ChatModel.ProtocolManager.Chats.Count > 1) {
                 Gtk.MessageDialog md = new Gtk.MessageDialog(
                     Frontend.MainWindow,
@@ -73,16 +74,22 @@ namespace Smuxi.Frontend.Gnome
                     return;
                 }
             }
-            
-            base.Close();
-            
-            Frontend.Session.CommandNetwork(
-                new CommandModel(
-                    Frontend.FrontendManager,
-                    ChatModel,
-                    "close"
-                )
-            );
+
+            ThreadPool.QueueUserWorkItem(delegate {
+                try {
+                    // no need to call base.Close() as CommandNetwork() will
+                    // deal with it
+                    Frontend.Session.CommandNetwork(
+                        new CommandModel(
+                            Frontend.FrontendManager,
+                            ChatModel,
+                            "close"
+                        )
+                    );
+                } catch (Exception ex) {
+                    Frontend.ShowException(ex);
+                }
+            });
         }
         
         private static string _(string msg)
