@@ -37,8 +37,17 @@ namespace Smuxi.Frontend.Gnome
     {
         public static Gdk.Pixbuf IconPixbuf { get; private set; }
         
-        static SessionChatView() {
-            IconPixbuf = new Gdk.Pixbuf(null, "session-chat.svg", 16, 16);
+        protected override Gtk.Image DefaultTabImage {
+            get {
+                return new Gtk.Image(IconPixbuf);
+            }
+        }
+
+        static SessionChatView()
+        {
+            IconPixbuf = Frontend.LoadIcon(
+                "smuxi-session-chat", 16, "session-chat_256x256.png"
+            );
         }
 
         public SessionChatView(ChatModel chat) : base(chat)
@@ -46,11 +55,8 @@ namespace Smuxi.Frontend.Gnome
             Trace.Call(chat);
             
             OutputMessageTextView.ShowMarkerline = false;
+            OutputMessageTextView.ShowTimestamps = false;
 
-            var tabImage = new Gtk.Image(IconPixbuf);
-            TabHBox.PackStart(tabImage, false, false, 2);
-            TabHBox.ShowAll();
-            
             Add(OutputScrolledWindow);
         }
         
