@@ -1,13 +1,7 @@
 /*
- * $Id: MainWindow.cs 192 2007-04-22 11:48:12Z meebey $
- * $URL: svn+ssh://svn.qnetp.net/svn/smuxi/smuxi/trunk/src/Frontend-GNOME/MainWindow.cs $
- * $Rev: 192 $
- * $Author: meebey $
- * $Date: 2007-04-22 13:48:12 +0200 (Sun, 22 Apr 2007) $
- *
  * Smuxi - Smart MUltipleXed Irc
  *
- * Copyright (c) 2007 Mirco Bauer <meebey@meebey.net>
+ * Copyright (c) 2007, 2010-2011 Mirco Bauer <meebey@meebey.net>
  *
  * Full GPL License: <http://www.gnu.org/licenses/gpl.txt>
  *
@@ -32,6 +26,7 @@ using System.Reflection;
 using Mono.Unix;
 using Smuxi.Common;
 using Smuxi.Engine;
+using Stfl;
 
 namespace Smuxi.Frontend.Stfl
 {
@@ -55,11 +50,29 @@ namespace Smuxi.Frontend.Stfl
                 return _UI;
             }
         }
-        
+
+        public string InputLabel {
+            get {
+                return this["input_label_text"];
+            }
+            set {
+                this["input_label_text"] = String.Format("{0} ", value);
+            }
+        }
+
+        public string NavigationLabel {
+            get {
+                return this["navigation_label_text"];
+            }
+            set {
+                this["navigation_label_text"] = value;
+            }
+        }
+
         public MainWindow() : base(null, "MainWindow.stfl")
         {
             _ChatViewManager = new ChatViewManager(this);
-            _Entry = new Entry(this);
+            _Entry = new Entry(this, _ChatViewManager);
             _UI = new StflUI(_ChatViewManager);
     	    Assembly asm = Assembly.GetExecutingAssembly();
     	    _ChatViewManager.Load(asm);
