@@ -51,8 +51,6 @@ namespace Smuxi.Server
 
         public static void Init(string[] args)
         {
-            Thread.CurrentThread.Name = "Main";
-            
             Engine.Engine.Init();
             string channel = (string)Engine.Engine.Config["Server/Channel"];
             string formatter = (string)Engine.Engine.Config["Server/Formatter"];
@@ -118,7 +116,10 @@ namespace Smuxi.Server
             _Logger.Info("Spawned remoting server with channel: "+channel+" formatter: "+formatter+" port: "+port);
 #endif            
             
-            Thread.Sleep(Timeout.Infinite);
+            Thread.CurrentThread.Join();
+#if LOG4NET
+            _Logger.Info("Shutting down remoting server...");
+#endif
         }
     }
 }
